@@ -44,6 +44,9 @@ setGeneric("export_all",
 setMethod("export_all", signature(object="familiarCollection"),
           function(object, dir_path=NULL, aggregate_results=waiver(), ...){
             
+            # Make sure the collection object is updated.
+            object <- update_object(object=object)
+            
             # Export feature selection variable importance
             fs_vimp <- export_fs_vimp(object=object,
                                       dir_path=dir_path)
@@ -278,137 +281,6 @@ setMethod(".export_to_file", signature(data="character", object="familiarCollect
 
 
 
-#####.apply_labels (data.table, familiarCollection)#####
-setMethod(".apply_labels", signature(data="data.table", object="familiarCollection"),
-          function(data, object){
-            browser()
-            # Return NULL for empty input
-            if(is_empty(data)) return(NULL)
-            
-            # Check if data has the expected class.
-            # if(!data.table::is.data.table(data)){
-            #   
-            #   if(is.list(data)){
-            #     # Get the names of the list elements.
-            #     element_names <- names(data)
-            #     
-            #     # Update underlying data
-            #     data <- lapply(data, .apply_labels, object=object)
-            #     
-            #     # Set the names
-            #     names(data) <- element_names
-            #     
-            #     return(data)
-            #   } else if(is.numeric(data) | is.character(data) | is.logical(data) | is.factor(data)){
-            #     return(data)
-            #     
-            #   } else {
-            #     stop(paste0("\"data\" is expected to be a \"data.frame\" or \"data.table\", or a list of these. Found: ", paste(class(data), sep=", "), "."))
-            #   }
-            # }
-
-            # Make sure that a local copy is updated
-            data <- data.table::copy(data)
-            
-            # Check which labels are present, based on column names
-            columns <- colnames(data)
-            
-            has_data_set           <- ifelse("data_set" %in% columns, TRUE, FALSE)
-            has_learner            <- ifelse("learner" %in% columns, TRUE, FALSE)
-            has_fs_method          <- ifelse("fs_method" %in% columns, TRUE, FALSE)
-            has_feature            <- ifelse(any(c("name", "feature_name_1", "feature_name_2", "feature") %in% columns), TRUE, FALSE)
-            has_risk_group         <- ifelse(any(c("risk_group", "risk_group_1", "risk_group_2", "reference_group") %in% columns), TRUE, FALSE)
-            has_multiclass_outcome <- ifelse(any(c("pos_class", "outcome") %in% columns) & object@outcome_type=="multinomial", TRUE, FALSE)
-            has_categorical_outcome <- ifelse(any(c("observed_outcome", "expected_outcome") %in% columns) & object@outcome_type %in% c("binomial", "multinomial"),
-                                              TRUE, FALSE)
-            has_evaluation_time <- any(c("evaluation_time", "eval_time") %in% columns) & object@outcome_type %in% c("survival", "competing_risk")
-            has_performance_metric <- any(c("metric") %in% columns)
-            
-            # Apply levels
-            if(has_data_set){
-              data$data_set <- factor(x=data$data_set,
-                                      levels=get_data_set_name_levels(x=object),
-                                      labels=get_data_set_names(x=object))
-            }
-            
-            if(has_learner){
-              data$learner <- factor(x=data$learner,
-                                     levels=get_learner_name_levels(x=object),
-                                     labels=get_learner_names(x=object))
-            }
-            
-            if(has_fs_method){
-              data$fs_method <- factor(x=data$fs_method,
-                                       levels=get_fs_method_name_levels(x=object),
-                                       labels=get_fs_method_names(x=object))
-            }
-            
-            if(has_feature){
-              for(curr_col_name in c("name", "feature_name_1", "feature_name_2", "feature")){
-                if(!is.null(data[[curr_col_name]])){
-                  data[[curr_col_name]] <- factor(x=data[[curr_col_name]],
-                                                  levels=get_feature_name_levels(x=object),
-                                                  labels=get_feature_names(x=object))
-                }
-              }
-            }
-            
-            if(has_risk_group){
-              for(curr_col_name in c("risk_group", "risk_group_1", "risk_group_2", "reference_group")){
-                if(!is.null(data[[curr_col_name]])){
-                  data[[curr_col_name]] <- factor(x=data[[curr_col_name]],
-                                                  levels=get_risk_group_name_levels(x=object),
-                                                  labels=get_risk_group_names(x=object))
-                }
-              }
-            }
-            
-            if(has_multiclass_outcome){
-              for(curr_col_name in c("pos_class", "outcome")){
-                if(!is.null(data[[curr_col_name]])){
-                  data[[curr_col_name]] <- factor(x=data[[curr_col_name]],
-                                                  levels=get_class_name_levels(x=object),
-                                                  labels=get_class_names(x=object))
-                }
-              }
-            }
-            
-            if(has_categorical_outcome){
-              # For confusion matrices.
-              for(curr_col_name in c("observed_outcome", "expected_outcome")){
-                if(!is.null(data[[curr_col_name]])){
-                  data[[curr_col_name]] <- factor(x=data[[curr_col_name]],
-                                                  levels=get_class_name_levels(x=object),
-                                                  labels=get_class_names(x=object))
-                }
-              }
-            }
-            
-            if(has_evaluation_time){
-              for(curr_col_name in c("evaluation_time", "eval_time")){
-                if(!is.null(data[[curr_col_name]])){
-                  data[[curr_col_name]] <- factor(x=data[[curr_col_name]],
-                                                  levels=sort(unique(data[[curr_col_name]])))
-                }
-              }
-            }
-            
-            if(has_performance_metric){
-              for(curr_col_name in c("metric")){
-                if(!is.null(data[[curr_col_name]])){
-                  data[[curr_col_name]] <- factor(x=data[[curr_col_name]],
-                                                  levels=sort(unique(data[[curr_col_name]])))
-                }
-              }
-            }
-            
-            # Drop unused levels
-            data <- droplevels(x=data)
-            
-            return(data)
-          })
-
-
 #####.apply_labels (familiarDataElement, familiarCollection)#####
 setMethod(".apply_labels", signature(data="familiarDataElement", object="familiarCollection"),
           function(data, object){
