@@ -6,22 +6,14 @@ run_feature_selection <- function(cl,
                                   verbose=TRUE){
 
   # Check which data object is required for performing feature selection.
-  fs_data_id <- .get_process_step_data_identifier(project_list=project_list,
+  fs_data_id <- .get_process_step_data_identifier(project_info=project_list,
                                                   process_step="fs")
   
   # Get feature selection methods that still need to be checked.
-  run_fs_methods <- .find_missing_feature_selection_data(proj_list=project_list,
+  run_fs_methods <- .find_missing_feature_selection_data(project_list=project_list,
                                                          settings=settings,
                                                          file_paths=file_paths)
 
-  # Check whether pre-processing has been conducted.
-  check_pre_processing(cl=cl,
-                       data_id=fs_data_id,
-                       file_paths=file_paths,
-                       project_id=project_list$project_id,
-                       message_indent=message_indent,
-                       verbose=verbose)
-
   # Get runs
   run_list <- .get_run_list(iteration_list=project_list$iter_list,
                             data_id=fs_data_id)
@@ -58,12 +50,10 @@ run_feature_selection <- function(cl,
                                progress_bar=verbose,
                                MoreArgs=list("fs_method"=curr_fs_method,
                                              "hpo_list"=hpo_list,
-                                             "proj_list"=project_list,
-                                             "settings"=settings,
-                                             "file_paths"=file_paths))
+                                             "settings"=settings))
     
     # Save to file
-    saveRDS(vimp_list, file=.get_feature_selection_data_filename(proj_list=project_list,
+    saveRDS(vimp_list, file=.get_feature_selection_data_filename(project_id = project_list$project_id,
                                                                  fs_method=curr_fs_method,
                                                                  file_paths=file_paths))
 
@@ -72,18 +62,16 @@ run_feature_selection <- function(cl,
                    indent=message_indent,
                    verbose=verbose)
   }
+  
+  invisible(TRUE)
 }
 
 
 
-compute_variable_importance <- function(run, fs_method, hpo_list, proj_list, settings, file_paths){
+compute_variable_importance <- function(run, fs_method, hpo_list, settings){
   # Function for calculating variable importance
-  
-  # Suppress NOTES due to non-standard evaluation in data.table
-  name <- NULL
-  
-  ############### Data preparation ################################################################
-  # Pre-process data
+
+  #### Data preparation --------------------------------------------------------
   
   # Data will be loaded at run time in .vimp.
   data <- methods::new("dataObject",
@@ -96,15 +84,15 @@ compute_variable_importance <- function(run, fs_method, hpo_list, proj_list, set
                        aggregate_on_load = FALSE,
                        outcome_info = create_outcome_info(settings=settings))
   
-  ############## Select parameters ################################################################
+  #### Select parameters -------------------------------------------------------
   parameter_list <- .find_hyperparameters_for_run(run=run,
                                                   hpo_list=hpo_list,
                                                   as_list=TRUE)
 
-  ############## Variable importance calculation ##################################################
+  #### Variable importance calculation -----------------------------------------
   
   # Load feature_info_list
-  feature_info_list <- get_feature_info_list(run=run)
+  feature_info_list <- .get_feature_info_list(run=run)
   
   # Create the variable importance method object or familiar model object to
   # compute variable importance with.
@@ -124,86 +112,92 @@ compute_variable_importance <- function(run, fs_method, hpo_list, proj_list, set
   
   # Find required features. Exclude the signature features at this point, as
   # these will have been dropped from the variable importance table.
-  required_features <- find_required_features(features=get_available_features(feature_info_list=feature_info_list,
-                                                                              exclude_signature=!is_multivariate),
-                                              feature_info_list=feature_info_list)
+  required_features <- get_required_features(x=data,
+                                             feature_info_list=feature_info_list,
+                                             exclude_signature=!is_multivariate)
   
   # Limit to required features.
   vimp_object@required_features <- required_features
   vimp_object@feature_info <- feature_info_list[required_features]
   
   # Compute variable importance.
-  vimp_table <- .vimp(object=vimp_object, data=data)
+  vimp_object <- .vimp(object=vimp_object, data=data)
   
-  # Post-processing on the variable importance data table
-  if(nrow(vimp_table) == 0){
-    # If the variable importance data table is empty, return an empty table
-    vimp_table$data_id   <- numeric(0)
-    vimp_table$run_id    <- numeric(0)
-    vimp_table$fs_method <- character(0)
-    
-  } else {
-    # Obtain the list of identifiers.
-    id_list <- .get_iteration_identifiers(run=run)
-    
-    # Add identifiers to variable importance data table
-    vimp_table$data_id   <- id_list$data
-    vimp_table$run_id    <- id_list$run
-    vimp_table$fs_method <- fs_method
-  }
-  
-  # Remove signature features from the table, and re-rank.
-  if(is_multivariate & nrow(vimp_table) > 0){
-    
-    # Identify signature features, if any.
-    signature_features <- names(feature_info_list)[sapply(feature_info_list, is_in_signature)]
-    
-    if(length(signature_features) > 0){
-      # Remove signature features.
-      vimp_table <- vimp_table[!name %in% signature_features]
-      
-      if(nrow(vimp_table) > 0){
-        # Re-rank features after removing signature features.
-        vimp_table[, "rank":=data.table::frank(rank, ties.method="min")]
-      }
-    }
-  }
-
-  # Generate the translation table for the selected set of features.
-  translation_table <- rank.get_decluster_translation_table(features=vimp_table$name,
-                                                            feature_info_list=feature_info_list[required_features])
-  
-  return(list("run_table"=run$run_table,
-              "fs_method"=fs_method,
-              "vimp"=vimp_table,
-              "translation_table"=translation_table))
+  return(vimp_object)
 }
 
 
 
-.find_missing_feature_selection_data <- function(proj_list, settings, file_paths){
+.find_missing_feature_selection_data <- function(project_list, settings, file_paths){
 
   # Suppress NOTES due to non-standard evaluation in data.table
   fs_method <- fs_file <- NULL
 
   # All feature selection methods
-  dt_fs <- data.table::data.table("fs_method"=settings$fs$fs_methods)
+  file_table <- data.table::data.table("fs_method"=settings$fs$fs_methods)
 
   # Add expected feature selection file names
-  dt_fs[,"fs_file":=.get_feature_selection_data_filename(proj_list=proj_list, fs_method=fs_method, file_paths=file_paths)]
+  file_table[, "fs_file":=.get_feature_selection_data_filename(project_id=project_list$project_id, 
+                                                               fs_method=fs_method,
+                                                               file_paths=file_paths)]
 
   # List files in directory
-  file_list <- normalizePath(list.files(path=file_paths$fs_dir, pattern=paste0(proj_list$project_id, "_fs_"), full.names=TRUE, recursive=TRUE), mustWork=FALSE)
+  file_list <- normalizePath(list.files(path=file_paths$fs_dir,
+                                        pattern=paste0(project_list$project_id, "_fs_"),
+                                        full.names=TRUE,
+                                        recursive=TRUE),
+                             mustWork=FALSE)
 
   # Remove files which have already been selected
-  dt_fs <- dt_fs[!fs_file %in% file_list, ]
-
-  return(dt_fs$fs_method)
+  file_table <- file_table[!fs_file %in% file_list, ]
 
+  return(file_table$fs_method)
 }
 
 
 
-.get_feature_selection_data_filename <- function(fs_method, proj_list, file_paths){
-  return(normalizePath(file.path(file_paths$fs_dir, paste0(proj_list$project_id, "_fs_", fs_method, ".RDS")), mustWork=FALSE))
+.get_feature_selection_data_filename <- function(fs_method, project_id, file_paths){
+  
+  return(normalizePath(file.path(file_paths$fs_dir,
+                                 paste0(project_id, "_fs_", fs_method, ".RDS")),
+                       mustWork=FALSE))
+}
+
+
+.retrieve_feature_selection_data <- function(fs_method, project_list, file_paths){
+  
+  # Iterate over the variable importance methods to select the correct files.
+  vimp_table_list <- lapply(fs_method, function(x, project_list, file_paths){
+    
+    # Attempt to read the object. This should produce a list of variable
+    # importance tables.
+    vimp_table_sub_list <- tryCatch(readRDS(.get_feature_selection_data_filename(fs_method=x,
+                                                                                 project_id=project_list$project_id,
+                                                                                 file_paths=file_paths)),
+                                    error=identity)
+    
+    # Check if there were issues reading the file.
+    if(inherits(vimp_table_sub_list, "error")) return(NULL)
+    
+    # Iterate over contents of the variable importance table list to update
+    # these to the latest definitions.
+    vimp_table_sub_list <- lapply(vimp_table_sub_list, function(x, project_id){
+      # Upgrade the object for backward compatibility.
+      if(!is(x, "vimpTable")) x <- as_vimp_table_object(x, project_id=project_id)
+      
+      # Update variable importance table objects to the most recent definition.
+      return(update_object(x))
+    },
+    project_id=project_list$project_id)
+    
+    # Return list of vimpTable objects.
+    return(vimp_table_sub_list)
+  },
+  project_list=project_list,
+  file_paths=file_paths)
+
+  # Update names
+  names(vimp_table_list) <- fs_method
+  
+  return(vimp_table_list)
 }
