@@ -1,70 +1,42 @@
-.create_hyperparameter_challenger_sets <- function(parameter_table,
-                                                   score_table,
+.create_hyperparameter_challenger_sets <- function(score_table,
                                                    parameter_list,
+                                                   parameter_table,
                                                    time_optimisation_model,
-                                                   smbo_iter,
-                                                   hyperparameter_learner,
-                                                   optimisation_function,
+                                                   score_optimisation_model,
                                                    acquisition_function,
                                                    n_max_bootstraps,
                                                    n_challengers=20L,
+                                                   smbo_iter,
                                                    measure_time,
                                                    random_admixture_fraction=0.20){
   
   # Suppress NOTES due to non-standard evaluation in data.table
-  param_id <- expected_train_time <- expected_improvement <- optimisation_score <- weight <- NULL 
+  param_id <- expected_train_time <- utility <- summary_score <- weight <- NULL 
   is_observed <- .NATURAL <- i.param_id <- NULL
   
   # Set the maximum number of local steps based on the number of randomisable
   # hyperparameters.
   n_max_local_steps <- 2 * sum(sapply(parameter_list, function(x) (x$randomise)))
   
-  # Compute the optimisation score. This creates a table with optimisation
-  # scores per bootstrap and parameter identifier.
-  optimisation_score_table <- .compute_hyperparameter_optimisation_score(score_table=score_table,
-                                                                         optimisation_function=optimisation_function)
-  
-  # Check for minimal size of optimisation_score_table to prevent errors in
-  # laGP.
-  if(hyperparameter_learner == "gaussian_process" & nrow(score_table) < 8){
-    logger.warning(paste0("Insufficient instances (<8) in the score table to derive an approximate Gaussian Process. ",
-                          "Switching to random search until suffcient instances are created."))
-    
-    hyperparameter_learner <- "random"
+  # Train hyperparameter optimisation model.
+  if(!model_is_trained(score_optimisation_model)){
+    score_optimisation_model <- .train(object=score_optimisation_model,
+                                       data=score_table,
+                                       parameter_data=parameter_table)
   }
   
-  # Check if all hyperparameters are invariant.
-  if(!hyperparameter_learner %in% c("random", "random_search")){
-    
-    # Get parameter names.
-    parameter_names <- setdiff(colnames(parameter_table),
-                               c("param_id", "run_id", "optimisation_score", "time_taken"))
-    
-    if(all(sapply(parameter_table[, mget(parameter_names)], is_singular_data))){
-      logger.warning(paste0("All hyperparameters are currently invariant. Switching to random search until non-invariant instances are created."))
-      
-      hyperparameter_learner <- "random"
-    }
-  }
-  
-  # Create a model to predict the optimisation score for a given parameter set.
-  score_optimisation_model <- .create_hyperparameter_score_optimisation_model(hyperparameter_learner=hyperparameter_learner,
-                                                                              score_table=optimisation_score_table,
-                                                                              parameter_table=parameter_table)
-  
-  # Find information regarding the dataset that has the highest optimisation
-  # score.
-  incumbent_set_data <- ..get_best_hyperparameter_set(optimisation_score_table=optimisation_score_table,
-                                                      acquisition_function=acquisition_function,
-                                                      n=1)
-  
-  # Get acquisition data.
-  acquisition_data <- ..get_acquisition_function_parameters(optimisation_score_table=optimisation_score_table,
-                                                            acquisition_function=acquisition_function,
-                                                            n_max_bootstraps=n_max_bootstraps)
+  # Find information regarding the dataset that has the highest
+  # optimisation score.
+  incumbent_set <- get_best_hyperparameter_set(score_table=score_table,
+                                               parameter_table=parameter_table,
+                                               optimisation_model=score_optimisation_model,
+                                               n_max_bootstraps=n_max_bootstraps,
+                                               n=1L)
   
   ##### Generate random sets----------------------------------------------------
-  if(hyperparameter_learner %in% c("random", "random_search")) random_admixture_fraction <- 1.0
+  
+  # Increase random admixture for random search.
+  if(is(score_optimisation_model, "familiarHyperparameterRandomSearch")) random_admixture_fraction <- 1.0
   
   # In general, draw about 10 times as much random sets then required.
   n_random_sets <- ceiling(n_challengers * random_admixture_fraction)
@@ -84,12 +56,11 @@
                                               use.names=TRUE))
   
   # Compute expected improvement.
-  if(!is_empty(random_sets) & !hyperparameter_learner %in% c("random", "random_search")){
-    random_sets[, "expected_improvement":=.compute_utility_value(parameter_set=.SD,
-                                                                 score_model=score_optimisation_model,
-                                                                 hyperparameter_learner=hyperparameter_learner,
-                                                                 acquisition_data=acquisition_data,
-                                                                 acquisition_function=acquisition_function)]
+  if(!is_empty(random_sets) & !is(score_optimisation_model, "familiarHyperparameterRandomSearch")){
+    random_sets[, "utility":=.compute_utility_value(parameter_set=.SD,
+                                                    score_model=score_optimisation_model,
+                                                    acquisition_data=incumbent_set,
+                                                    acquisition_function=acquisition_function)]
   }
   
   # Compute expect train time.
@@ -105,13 +76,15 @@
   # Find local sets
   local_sets <- list()
   
-  if(n_local_sets > 0 & !hyperparameter_learner %in% c("random", "random_search")){
+  if(n_local_sets > 0 & !is(score_optimisation_model, "familiarHyperparameterRandomSearch")){
     
     # Create a table of summary scores to identify the best sets.
-    summary_score_table <- metric.summarise_optimisation_score(score_table=optimisation_score_table, method=acquisition_function)
-    
+    summary_score_table <- .compute_hyperparameter_summary_score(score_table=score_table,
+                                                                 parameter_table=parameter_table,
+                                                                 optimisation_model=score_optimisation_model)
+                                                                 
     # Find the most promising samples by comparing against the median.
-    summary_score_table[, "weight":=optimisation_score - stats::median(optimisation_score)]
+    summary_score_table[, "weight":=summary_score - stats::median(summary_score)]
     
     # If all parameters have the same optimisation score (an edge case), assign
     # weight 1 to each, that is, all hyperparameter sets are as likely to be
@@ -123,7 +96,7 @@
     
     # Select the hyperparameter ids to sample, starting with the most promising
     # samples first.
-    summary_score_table <- summary_score_table[, "n_select":=round(2 * n_local_sets * weight^2/sum(weight^2))][order(-optimisation_score )]
+    summary_score_table <- summary_score_table[, "n_select":=round(2 * n_local_sets * weight^2/sum(weight^2))][order(-summary_score )]
     selected_best_parameter_id <- unlist(mapply(function(x, n) (rep(x, times=n)), x=summary_score_table$param_id, n=summary_score_table$n_select))
     
     # Perform a local search.
@@ -160,11 +133,10 @@
   
   if(!is_empty(local_sets)){
     # Compute expected improvement.
-    local_sets[, "expected_improvement":=.compute_utility_value(parameter_set=.SD,
-                                                                 score_model=score_optimisation_model,
-                                                                 hyperparameter_learner=hyperparameter_learner,
-                                                                 acquisition_data=acquisition_data,
-                                                                 acquisition_function=acquisition_function)]
+    local_sets[, "utility":=.compute_utility_value(parameter_set=.SD,
+                                                   score_model=score_optimisation_model,
+                                                   acquisition_data=incumbent_set,
+                                                   acquisition_function=acquisition_function)]
     
     # Compute expected train time.
     local_sets[, "expected_train_time":=.compute_expected_train_time(parameter_set=.SD,
@@ -177,16 +149,16 @@
   local_challenger_sets <- list()
   random_challenger_sets <- list()
   
-  if(hyperparameter_learner %in% c("random", "random_search")){
+  if(is(score_optimisation_model, "familiarHyperparameterRandomSearch")){
     # Preferentially select in the following order:
     # 1. Random sets that were not observed before.
     # 2. Random sets that were observed before.
     
     # Remove incumbent set.
-    if(!is_empty(random_sets)) random_sets <- random_sets[!parameter_table[param_id == incumbent_set_data$param_id], on=.NATURAL]
+    if(!is_empty(random_sets)) random_sets <- random_sets[!parameter_table[param_id == incumbent_set$param_id], on=.NATURAL]
     
     # Select only parameter sets that can be evaluated within the allowed time.
-    if(measure_time & !is_empty(random_sets)) random_sets <- random_sets[expected_train_time < acquisition_data$max_time]
+    if(measure_time & !is_empty(random_sets)) random_sets <- random_sets[expected_train_time < incumbent_set$max_time]
     
     # Consider random sets.
     if(!is_empty(random_sets)){
@@ -240,10 +212,10 @@
     # 4. Random sets that were observed before, with highest utility first.
     
     # First, remove incumbent.
-    if(!is_empty(local_sets)) local_sets <- local_sets[!parameter_table[param_id == incumbent_set_data$param_id], on=.NATURAL]
+    if(!is_empty(local_sets)) local_sets <- local_sets[!parameter_table[param_id == incumbent_set$param_id], on=.NATURAL]
     
     # Select only parameter sets that can be evaluated within the allowed time.
-    if(measure_time & !is_empty(local_sets)) local_sets <- local_sets[expected_train_time < acquisition_data$max_time]
+    if(measure_time & !is_empty(local_sets)) local_sets <- local_sets[expected_train_time < incumbent_set$max_time]
     
     # Consider local sets.
     if(!is_empty(local_sets)){
@@ -256,7 +228,7 @@
       if(any(!local_sets$is_observed)){
         # Select up to n_local_sets of locally sampled sets that were not
         # selected previously.
-        selected_sets <- head(local_sets[is_observed==FALSE][order(-expected_improvement)], n=n_local_sets)
+        selected_sets <- head(local_sets[is_observed==FALSE][order(-utility)], n=n_local_sets)
         
         # Add to challenger sets and reduce the number of challengers to select.
         local_challenger_sets <- list(selected_sets)
@@ -268,7 +240,7 @@
       if(any(local_sets$is_observed) & n_local_sets > 0){
         # Select up to n_local_sets of locally samples sets that were observed
         # selected previously.
-        selected_sets <- head(local_sets[is_observed==TRUE][order(-expected_improvement)], n=n_local_sets)
+        selected_sets <- head(local_sets[is_observed==TRUE][order(-utility)], n=n_local_sets)
         
         # Add to challenger sets and reduce the number of challengers to select.
         local_challenger_sets <- c(local_challenger_sets, list(selected_sets))
@@ -281,17 +253,17 @@
       local_challenger_sets <- data.table::rbindlist(local_challenger_sets, use.names=TRUE)
       
       # Remove expected improvement and time from challenger sets.
-      local_challenger_sets[, ":="("expected_improvement"=NULL,
+      local_challenger_sets[, ":="("utility"=NULL,
                                    "expected_train_time"=NULL,
                                    "is_observed"=NULL)]
     }
     
     # Remove incumbent and already selected challenger sets.
-    if(!is_empty(random_sets)) random_sets <- random_sets[!parameter_table[param_id == incumbent_set_data$param_id], on=.NATURAL]
+    if(!is_empty(random_sets)) random_sets <- random_sets[!parameter_table[param_id == incumbent_set$param_id], on=.NATURAL]
     if(!is_empty(random_sets) & !is_empty(local_challenger_sets) > 0) random_sets <- random_sets[!local_challenger_sets, on=.NATURAL]
     
     # Select only parameter sets that can be evaluated within the allowed time.
-    if(measure_time & !is_empty(random_sets)) random_sets <- random_sets[expected_train_time < acquisition_data$max_time]
+    if(measure_time & !is_empty(random_sets)) random_sets <- random_sets[expected_train_time < incumbent_set$max_time]
     
     # Update n_random_sets to compensate for selected challengers.
     n_random_sets <- min(c(n_challengers, n_random_sets))
@@ -307,7 +279,7 @@
       if(any(!random_sets$is_observed) & n_random_sets > 0){
         # Select up to n_random_sets of random samples sets that were not
         # selected previously.
-        selected_sets <- head(random_sets[is_observed==FALSE][order(-expected_improvement)], n=n_random_sets)
+        selected_sets <- head(random_sets[is_observed==FALSE][order(-utility)], n=n_random_sets)
         
         # Add to challenger sets and reduce the number of challengers to select.
         random_challenger_sets <- list(selected_sets)
@@ -319,7 +291,7 @@
       if(any(random_sets$is_observed) & n_random_sets > 0){
         # Select up to n_random_sets of random samples sets that were observed
         # selected previously.
-        selected_sets <- head(random_sets[is_observed==TRUE][order(-expected_improvement)], n=n_random_sets)
+        selected_sets <- head(random_sets[is_observed==TRUE][order(-utility)], n=n_random_sets)
         
         # Add to challenger sets and reduce the number of challengers to select.
         random_challenger_sets <- c(random_challenger_sets, list(selected_sets))
@@ -332,7 +304,7 @@
       random_challenger_sets <- data.table::rbindlist(random_challenger_sets, use.names=TRUE)
       
       # Remove expected improvement and time from challenger sets.
-      random_challenger_sets[, ":="("expected_improvement"=NULL,
+      random_challenger_sets[, ":="("utility"=NULL,
                                     "expected_train_time"=NULL,
                                     "is_observed"=NULL)]
     }
@@ -365,94 +337,26 @@
 
 
 
-.create_hyperparameter_score_optimisation_model <- function(hyperparameter_learner,
-                                                            score_table,
-                                                            parameter_table){
-  
-  if(hyperparameter_learner == "random_forest"){
-    model <- ..hyperparameter_random_forest_optimisation_score_learner(score_table=score_table,
-                                                                       parameter_table=parameter_table)
-    
-  } else if(hyperparameter_learner == "gaussian_process"){
-    model <- ..hyperparameter_gaussian_process_optimisation_score_learner(score_table=score_table,
-                                                                          parameter_table=parameter_table)
-    
-  } else if(hyperparameter_learner %in% c("bayesian_additive_regression_trees", "bart")){
-    model <- ..hyperparameter_bart_optimisation_score_learner(score_table=score_table,
-                                                              parameter_table=parameter_table)
-    
-  } else if(hyperparameter_learner %in% c("random", "random_search")){
-    model <- NULL
-    
-  } else {
-    ..error_reached_unreachable_code(paste0(".create_hyperparameter_score_optimisation_model: hyperparameter_learner was not recognised: ", hyperparameter_learner))
-  }
-  
-  return(model)
-}
-  
-
-
 .create_hyperparameter_time_optimisation_model <- function(hyperparameter_learner="random_forest",
                                                            score_table,
-                                                           parameter_table){
+                                                           parameter_table,
+                                                           optimisation_function){
   
+  # Check that score_table contains optimisation scores.
+  if(!"optimisation_score" %in% colnames(score_table)){
+    score_table <- .compute_hyperparameter_optimisation_score(score_table=score_table,
+                                                              optimisation_function=optimisation_function)
+  }
+  
+  # Train model to predict process time.
   model <- ..hyperparameter_random_forest_time_learner(score_table=score_table,
                                                        parameter_table=parameter_table)
   
-  # if(hyperparameter_learner == "random_forest"){
-  #   model <- ..hyperparameter_random_forest_time_learner(score_table=score_table,
-  #                                                        parameter_table=parameter_table)
-  # } else {
-  #   ..error_reached_unreachable_code(paste0(".create_hyperparameter_time_optimisation_model: hyperparameter_learner was not recognised: ", hyperparameter_learner))
-  # }
-  
   return(model)
 }
 
 
 
-..hyperparameter_random_forest_optimisation_score_learner <- function(score_table, parameter_table){
-  
-  # Suppress NOTES due to non-standard evaluation in data.table
-  optimisation_score <- NULL
-  
-  # Check if package is installed
-  require_package(x="ranger", purpose="to perform model-based hyperparameter optimisation")
-  
-  # Merge score and parameter data tables on param id.
-  joint_table <- merge(x=score_table,
-                       y=parameter_table,
-                       by="param_id",
-                       all=FALSE)
-  
-  # Replace NA entries with the minimum optimisation score.
-  joint_table[is.na(optimisation_score), optimisation_score:=-1.0]
-  
-  # Get parameter names.
-  parameter_names <- setdiff(colnames(parameter_table),
-                             c("param_id", "run_id", "optimisation_score", "time_taken"))
-  
-  # Hyperparameters for the random forest.
-  n_tree <- 400
-  n_train <- nrow(joint_table)
-  sample_fraction <- max(c(0.3, min(c(1, 1/(0.025*n_train)))))
-  
-  # Parse formula.
-  formula <- stats::reformulate(termlabels=parameter_names,
-                                response="optimisation_score")
-  
-  # Train random forest.
-  rf_model <- ranger::ranger(formula,
-                             data=joint_table,
-                             num.trees=n_tree,
-                             num.threads=1L,
-                             sample.fraction=sample_fraction,
-                             verbose=FALSE)
-  
-  return(rf_model)
-}
-
 
 
 ..hyperparameter_random_forest_time_learner <- function(score_table, parameter_table){
@@ -508,263 +412,101 @@
 
 
 
-..hyperparameter_gaussian_process_optimisation_score_learner <- function(score_table, parameter_table){
-  # Suppress NOTES due to non-standard evaluation in data.table
-  optimisation_score <- NULL
-  
-  # Merge score and parameter data tables on param id.
-  joint_table <- merge(x=score_table,
-                       y=parameter_table,
-                       by="param_id",
-                       all=FALSE)
-  
-  # Replace NA entries with the minimum optimisation score.
-  joint_table[is.na(optimisation_score), optimisation_score:=-1.0]
-  
-  # Get parameter names.
-  parameter_names <- setdiff(colnames(parameter_table),
-                             c("param_id", "run_id", "optimisation_score", "time_taken"))
-  
-  # Get training data and response.
-  x <- joint_table[, mget(parameter_names)]
-  
-  # Encode categorical variables.
-  x_encoded <- encode_categorical_variables(data=x,
-                                            object=NULL,
-                                            encoding_method="dummy",
-                                            drop_levels=FALSE)$encoded_data
-  
-  # Get optimisation score as response.
-  y <- joint_table$optimisation_score
-  
-  return(list("X"=as.matrix(x_encoded),
-              "Z"=y))
-}
-
-
-
-..hyperparameter_bart_optimisation_score_learner <- function(score_table, parameter_table){
-  # Suppress NOTES due to non-standard evaluation in data.table
-  optimisation_score <- NULL
-  
-  # Check if package is installed
-  require_package(x="BART", purpose="to predict process time of hyperparameter sets")
-  
-  # Merge score and parameter data tables on param id.
-  joint_table <- merge(x=score_table,
-                       y=parameter_table,
-                       by="param_id",
-                       all=FALSE)
-  
-  # Replace NA entries with the minimum optimisation score.
-  joint_table[is.na(optimisation_score), optimisation_score:=-1.0]
-  
-  # Get parameter names.
-  parameter_names <- setdiff(colnames(parameter_table),
-                             c("param_id", "run_id", "optimisation_score", "time_taken"))
-  
-  # Get training data and response.
-  x <- joint_table[, mget(parameter_names)]
-  
-  # Drop invariant columns.
-  invariant_columns <- sapply(x, is_singular_data)
-  parameter_names <- parameter_names[!invariant_columns]
-  x <- joint_table[, mget(parameter_names)]
-  
-  # Encode categorical variables.
-  x_encoded <- encode_categorical_variables(data=x,
-                                            object=NULL,
-                                            encoding_method="dummy",
-                                            drop_levels=FALSE)$encoded_data
-  
-  # Get optimisation score as response.
-  y <- joint_table$optimisation_score
-  
-  # Create model. Note that prediction by BART is expensive for large ndpost.
-  quiet(bart_model <- BART::wbart(x.train=data.frame(x_encoded), y.train=y, ntree=100, ndpost=50))
-  
-  return(bart_model)
-}
-
-
 .compute_utility_value <- function(parameter_set,
                                    score_model,
-                                   hyperparameter_learner,
                                    acquisition_data,
                                    acquisition_function){
-  # Predict score and compute .
-  if(hyperparameter_learner == "random_forest"){
-    # Check if package is installed
-    require_package(x="ranger", purpose="to perform model-based hyperparameter optimisation")
-    
-    # Compute score per decision tree in the random forest.
-    predicted_scores <- predict(score_model,
-                                data=parameter_set,
-                                predict.all=TRUE,
-                                num.threads=1L,
-                                verbose=FALSE)$predictions
-    
-    # Compute utility scores.
-    utility_scores <- apply(predicted_scores,
-                            MARGIN=1,
-                            ..compute_utility_score,
-                            acquisition_function=acquisition_function,
-                            acquisition_data=acquisition_data,
-                            n_parameters=ncol(parameter_set))
-    
-  } else if(hyperparameter_learner == "gaussian_process") {
-    # Check if package is installed
-    require_package(x="laGP", purpose="to perform model-based hyperparameter optimisation")
-    
-    # Encode categorical variables.
-    x_encoded <- encode_categorical_variables(data=parameter_set,
-                                              object=NULL,
-                                              encoding_method="dummy",
-                                              drop_levels=FALSE)$encoded_data
-    
-    # Update the end-size parameter, if the number of instances is smaller than
-    # the default of 51. Note that we already capture end_size of 7 or smaller
-    # in the parent function.
-    end_size <- ifelse(nrow(score_model$X) < 51, nrow(score_model$X) - 1, 50)
-    
-    # Infer scores for the hyperparameters.
-    quiet(predicted_scores <- laGP::aGP(X=score_model$X,
-                                        Z=score_model$Z,
-                                        XX=as.matrix(x_encoded),
-                                        end=end_size))
-    
-    # Compute utility_scores
-    utility_scores <- mapply(..compute_utility_score,
-                             mu=predicted_scores$mean,
-                             sigma=sqrt(predicted_scores$var),
-                             MoreArgs=list("acquisition_function"=acquisition_function,
-                                           "acquisition_data"=acquisition_data,
-                                           "n_parameters"=ncol(parameter_set)))
-    
-  } else if(hyperparameter_learner %in% c("bayesian_additive_regression_trees", "bart")){
-    
-    # Check if package is installed
-    require_package(x="BART", purpose="to perform model-based hyperparameter optimisation")
-    
-    # Drop columns not in the model object.
-    modelled_parameters <- colnames(score_model$varcount)
-    
-    # Encode categorical variables.
-    x_encoded <- encode_categorical_variables(data=parameter_set,
-                                              object=NULL,
-                                              encoding_method="dummy",
-                                              drop_levels=FALSE)$encoded_data
-    
-    # Get predicted values.
-    quiet(predicted_scores <- predict(score_model, data.frame(x_encoded[, mget(modelled_parameters)])))
-    
-    # Compute utility scores.
-    utility_scores <- apply(predicted_scores,
-                            MARGIN=2,
-                            ..compute_utility_score,
-                            acquisition_function=acquisition_function,
-                            acquisition_data=acquisition_data,
-                            n_parameters=ncol(parameter_set))
-    
-  } else if(hyperparameter_learner %in% c("random", "random_search")){
-    # Utility scores are not used for utility scores.
-    utility_scores <- rep_len(NA_real_, nrow(parameter_set))
-    
-  } else {
-    ..error_reached_unreachable_code(paste0(".compute_utility_value: hyperparameter_learner was not recognised: ", hyperparameter_learner))
-  }
+  
+  # Check that the score_model is trained, and return NA if not.
+  if(!model_is_trained(score_model)) return(rep_len(NA_real_, nrow(parameter_set)))
+  
+  # Dispatch to acquisition functions.
+  acquisition_FUN <- switch(acquisition_function,
+                            "improvement_probability" = acquisition_improvement_probability,
+                            "improvement_empirical_probability" = acquisition_improvement_empirical_probability,
+                            "expected_improvement" = acquisition_expected_improvement,
+                            "upper_confidence_bound" = acquisition_upper_confidence_bound,
+                            "bayes_upper_confidence_bound" = acquisition_bayes_upper_confidence_bound)
+  
+  
+  utility_scores <- acquisition_FUN(object=score_model,
+                                    parameter_data=parameter_set,
+                                    acquisition_data=acquisition_data)
   
   return(utility_scores)
 }
 
 
 
-..compute_utility_score <- function(x=NULL, mu=NULL, sigma=NULL, acquisition_function, acquisition_data, n_parameters){
+acquisition_improvement_probability <- function(object,
+                                                parameter_data,
+                                                acquisition_data){
+  # The definition of probability of improvement is found in Shahriari, B.,
+  # Swersky, K., Wang, Z., Adams, R. P. & de Freitas, N. Taking the Human Out
+  # of the Loop: A Review of Bayesian Optimization. Proc. IEEE 104, 148–175
+  # (2016)
   
-  if(is.null(mu)) mu <- mean(x)
-  if(is.null(sigma)) sigma <- stats::sd(x)
+  # Get the score estimate from the incumbent hyperparameter set.
+  tau <- acquisition_data$score_estimate
   
-  if(!is.null(x)) if(all(!is.finite(x))) return(0.0)
-  if(!is.finite(mu)) return(0.0)
-  if(!is.finite(sigma)) return(0.0)
+  # Predict the mean and standard deviation for the parameter sets in parameter
+  # data.
+  prediction_table <- .predict(object=object,
+                               data=parameter_data,
+                               type="sd")
   
-  if(acquisition_function == "improvement_probability"){
-    # The definition of probability of improvement is found in Shahriari, B.,
-    # Swersky, K., Wang, Z., Adams, R. P. & de Freitas, N. Taking the Human Out
-    # of the Loop: A Review of Bayesian Optimization. Proc. IEEE 104, 148–175
-    # (2016)
+  # Compute the probability of improvement.
+  alpha <- stats::pnorm((prediction_table$mu - tau) / prediction_table$sigma)
+  
+  return(alpha)
+}
+
+
+
+acquisition_improvement_empirical_probability <- function(object,
+                                                          parameter_data,
+                                                          acquisition_data){
+  # The definition of probability of improvement is found in Shahriari, B.,
+  # Swersky, K., Wang, Z., Adams, R. P. & de Freitas, N. Taking the Human Out
+  # of the Loop: A Review of Bayesian Optimization. Proc. IEEE 104, 148–175
+  # (2016)
+  
+  # Get the incumbent score.
+  tau <- acquisition_data$score_estimate
+  
+  if("raw" %in% get_prediction_type(object)){
+    # Compute the empirical probability of improvement.
     
-    # Get the incumbent score.
-    tau <- acquisition_data$tau
-    
-    # Compute the probability of improvement.
-    alpha <- stats::pnorm((mu - tau) / sigma)
-    
-  } else if(acquisition_function == "improvement_empirical_probability"){
-    
-    # Get the incumbent score.
-    tau <- acquisition_data$tau
-    
-    if(!is.null(x)){
-      # Compute the empirical probability of improvement.
-      alpha <- sum(x > tau) / length(x)
-      
-    } else {
-      # Compute the stochastic probability of improvement.
-      alpha <- stats::pnorm((mu - tau) / sigma)
+    # Define helper function.
+    .acquisition_improvement_empirical_probability <- function(x, tau){
+      return(sum(x > tau) / length(x))
     }
     
-  } else if(acquisition_function == "expected_improvement"){
-    # The definition of expected improvement is found in Shahriari, B., Swersky,
-    # K., Wang, Z., Adams, R. P. & de Freitas, N. Taking the Human Out of the
-    # Loop: A Review of Bayesian Optimization. Proc. IEEE 104, 148–175 (2016).
+    # Predict raw data for the hyperparameter sets.
+    prediction_table <- .predict(object=object,
+                                 data=parameter_data,
+                                 type="raw")
     
-    # Get the incumbent score.
-    tau <- acquisition_data$tau
+    # Drop parameter id. This should leave only raw data.
+    prediction_table[, "param_id":=NULL]
     
-    # Compute a z-score.
-    if(sigma > 0){
-      z <- (mu - tau) / sigma
-    } else {
-      return(0.0)
-    }
-    
-    # Compute the expected improvement.
-    alpha <- (mu - tau) * stats::pnorm(z) + sigma * stats::dnorm(z)
-    
-  } else if(acquisition_function == "upper_confidence_bound"){
-    # The definition of upper confidence bound is adapted from Srinivas, N.,
-    # Krause, A., Kakade, S. M. & Seeger, M. W. Information-Theoretic Regret
-    # Bounds for Gaussian Process Optimization in the Bandit Setting. IEEE
-    # Trans. Inf. Theory 58, 3250–3265 (2012).
-    
-    # Compute the beta parameter.
-    beta <- 2/5 * log(5/3 * n_parameters * (acquisition_data$t + 1)^2 * pi^2)
-    
-    # Compute alpha
-    alpha <- mu + beta * sigma
-    
-  } else if(acquisition_function == "bayes_upper_confidence_bound"){
-    # The definition of the Bayesian upper confidence bound is adapted from
-    # Kaufmann, E., Cappé, O. & Garivier, A. On Bayesian upper confidence bounds
-    # for bandit problems. in Artificial intelligence and statistics 592–600
-    # (2012).
-    
-    # Compute the quantile we are interested in.
-    q <- 1.0 - 1.0 / (acquisition_data$t + 1)
-    
-    if(!is.null(x)){
-      # Compute empiric alpha.
-      alpha <- stats::quantile(x=x,
-                               probs=q,
-                               names=FALSE)
-    } else {
-      # Stochastic alpha.
-      alpha <- stats::qnorm(q, mean=mu, sd=sigma)
-    }
+    # Compute utility.
+    alpha <- apply(prediction_table,
+                   MARGIN=1,
+                   .acquisition_improvement_empirical_probability,
+                   tau=tau)
     
   } else {
-    ..error_reached_unreachable_code(paste0("hpo.acquisition_function: unknown acquisition function: ", acquisition_function))
+    # Compute the stochastic probability of improvement.
+    
+    # Predict the mean and standard deviation for the parameter sets in parameter
+    # data.
+    prediction_table <- .predict(object=object,
+                                 data=parameter_data,
+                                 type="sd")
+    
+    # Stochastic values
+    alpha <- stats::pnorm((prediction_table$mu - tau) / prediction_table$sigma)
   }
   
   return(alpha)
@@ -772,6 +514,96 @@
 
 
 
+acquisition_expected_improvement <- function(object,
+                                             parameter_data,
+                                             acquisition_data){
+  # The definition of expected improvement is found in Shahriari, B., Swersky,
+  # K., Wang, Z., Adams, R. P. & de Freitas, N. Taking the Human Out of the
+  # Loop: A Review of Bayesian Optimization. Proc. IEEE 104, 148–175 (2016).
+  
+  # Get the incumbent score.
+  tau <- acquisition_data$score_estimate
+  
+  # Predict the mean and standard deviation for the parameter sets in parameter
+  # data.
+  prediction_table <- .predict(object=object,
+                               data=parameter_data,
+                               type="sd")
+  
+  # Compute a z-score.
+  z <- (prediction_table$mu - tau) / prediction_table$sigma
+
+  # Compute the expected improvement.
+  alpha <- (prediction_table$mu - tau) * stats::pnorm(z) + prediction_table$sigma * stats::dnorm(z)
+  
+  return(alpha)
+}
+
+
+
+acquisition_upper_confidence_bound <- function(object,
+                                               parameter_data,
+                                               acquisition_data){
+  # The definition of upper confidence bound is adapted from Srinivas, N.,
+  # Krause, A., Kakade, S. M. & Seeger, M. W. Information-Theoretic Regret
+  # Bounds for Gaussian Process Optimization in the Bandit Setting. IEEE
+  # Trans. Inf. Theory 58, 3250–3265 (2012).
+  
+  # Find the number hyperparameters.
+  n_parameters <- length(object@target_hyperparameters)
+  
+  # Predict the mean and standard deviation for the parameter sets in parameter
+  # data.
+  prediction_table <- .predict(object=object,
+                               data=parameter_data,
+                               type="sd")
+  
+  # Compute the beta parameter.
+  beta <- 2/5 * log(5/3 * n_parameters * (acquisition_data$t + 1)^2 * pi^2)
+  
+  # Compute alpha
+  alpha <- prediction_table$mu + beta * prediction_table$sigma
+  
+  return(alpha)
+}
+  
+
+
+acquisition_bayes_upper_confidence_bound <- function(object,
+                                                     parameter_data,
+                                                     acquisition_data){
+  # The definition of the Bayesian upper confidence bound is adapted from
+  # Kaufmann, E., Cappé, O. & Garivier, A. On Bayesian upper confidence bounds
+  # for bandit problems. in Artificial intelligence and statistics 592–600
+  # (2012).
+  
+  # Compute the quantile we are interested in.
+  q <- 1.0 - 1.0 / (acquisition_data$t + 1)
+  
+  if("percentile" %in% get_prediction_type(object)){
+    # Compute empiric alpha.
+    alpha <- .predict(object=object,
+                      data=parameter_data,
+                      type="percentile",
+                      percentile=q)$percentile
+    
+  } else {
+    # Predict the mean and standard deviation for the parameter sets in parameter
+    # data.
+    prediction_table <- .predict(object=object,
+                                 data=parameter_data,
+                                 type="sd")
+    
+    # Stochastic alpha.
+    alpha <- stats::qnorm(q, mean=prediction_table$mu, sd=prediction_table$sigma)
+  }
+  
+  return(alpha)
+}
+
+
+
+
 .compute_expected_train_time <- function(parameter_set,
                                          time_model){
   
@@ -805,30 +637,10 @@
 
 
 .get_available_hyperparameter_learners <- function(){
-  return(c("random_forest",
-           "gaussian_process",
-           "bayesian_additive_regression_trees", 
-           "bart",
-           "random",
-           "random_search"))
-}
-
-
-
-.required_packages_hyperparameter_learner <- function(x){
-  
-  learner_packages <- NULL
-  if(x == "random_forest"){
-    learner_packages <- "ranger"
-    
-  } else if(x == "gaussian_process"){
-    learner_packages <- "laGP"
-    
-  } else if(x %in% c("bayesian_additive_regression_trees", "bart")){
-    learner_packages <- "BART"
-  }
-  
-  return(learner_packages)
+  return(c(.get_available_ranger_hyperparameter_learners(),
+           .get_available_lagp_hyperparameter_learners(),
+           .get_available_bart_hyperparameter_learners(),
+           .get_available_random_search_hyperparameter_learners()))
 }
 
 
