@@ -155,7 +155,7 @@ testthat::test_that("SVM model trained correctly", {
 testthat::test_that("SVM model has no variable importance", {
   
   # Extract the variable importance table.
-  vimp_table <- familiar:::..vimp(good_model)
+  vimp_table <- familiar:::get_vimp_table(good_model)
   
   # Expect that the vimp table is empty.
   testthat::expect_equal(is_empty(vimp_table), TRUE)
@@ -168,7 +168,7 @@ testthat::test_that("SVM model can train on wide data", {
   testthat::expect_equal(familiar:::model_is_trained(wide_model), TRUE)
   
   # Variable importance table is absent
-  testthat::expect_equal(is_empty(familiar:::..vimp(wide_model)), TRUE)
+  testthat::expect_equal(familiar:::is_empty(familiar:::get_vimp_table(wide_model)), TRUE)
   
   # Valid predictions.
   testthat::expect_equal(familiar:::any_predictions_valid(familiar:::.predict(wide_model, wide_data), outcome_type=wide_data@outcome_type), TRUE)
@@ -222,7 +222,7 @@ testthat::test_that("SVM model trained correctly", {
 testthat::test_that("SVM model has no variable importance", {
   
   # Extract the variable importance table.
-  vimp_table <- familiar:::..vimp(good_model)
+  vimp_table <- familiar:::get_vimp_table(good_model)
   
   # Expect that the vimp table is empty.
   testthat::expect_equal(is_empty(vimp_table), TRUE)
@@ -235,7 +235,7 @@ testthat::test_that("SVM model can train on wide data", {
   testthat::expect_equal(familiar:::model_is_trained(wide_model), TRUE)
   
   # Variable importance table is absent.
-  testthat::expect_equal(is_empty(familiar:::..vimp(wide_model)), TRUE)
+  testthat::expect_equal(familiar:::is_empty(familiar:::get_vimp_table(wide_model)), TRUE)
   
   # Valid predictions.
   testthat::expect_equal(familiar:::any_predictions_valid(familiar:::.predict(wide_model, wide_data), outcome_type=wide_data@outcome_type), TRUE)
@@ -286,7 +286,7 @@ testthat::test_that("SVM model trained correctly", {
 testthat::test_that("SVM model has no variable importance", {
   
   # Extract the variable importance table.
-  vimp_table <- familiar:::..vimp(good_model)
+  vimp_table <- familiar:::get_vimp_table(good_model)
   
   # Expect that the vimp table is empty.
   testthat::expect_equal(is_empty(vimp_table), TRUE)
@@ -299,7 +299,7 @@ testthat::test_that("SVM model can train on wide data", {
   testthat::expect_equal(familiar:::model_is_trained(wide_model), TRUE)
   
   # Variable importance table is absent.
-  testthat::expect_equal(is_empty(familiar:::..vimp(wide_model)), TRUE)
+  testthat::expect_equal(is_empty(familiar:::get_vimp_table(wide_model)), TRUE)
   
   # Valid predictions.
   testthat::expect_equal(familiar:::any_predictions_valid(familiar:::.predict(wide_model, wide_data), outcome_type=wide_data@outcome_type), TRUE)
@@ -351,7 +351,7 @@ testthat::test_that("SVM model trained correctly", {
 testthat::test_that("SVM model has no variable importance", {
   
   # Extract the variable importance table.
-  vimp_table <- familiar:::..vimp(good_model)
+  vimp_table <- familiar:::get_vimp_table(good_model)
   
   # Expect that the vimp table is empty.
   testthat::expect_equal(is_empty(vimp_table), TRUE)
@@ -364,7 +364,7 @@ testthat::test_that("SVM model can train on wide data", {
   testthat::expect_equal(familiar:::model_is_trained(wide_model), TRUE)
   
   # Variable importance table is empty.
-  testthat::expect_equal(is_empty(familiar:::..vimp(wide_model)), TRUE)
+  testthat::expect_equal(familiar:::is_empty(familiar:::get_vimp_table(wide_model)), TRUE)
   
   # Valid predictions can be made.
   testthat::expect_equal(familiar:::any_predictions_valid(familiar:::.predict(wide_model, wide_data), outcome_type=wide_data@outcome_type), TRUE)
