@@ -93,7 +93,7 @@ testthat::test_that("Extreme gradient boosting regression model trained correctl
 testthat::test_that("Extreme gradient boosting regression model has variable importance", {
   
   # Extract the variable importance table.
-  vimp_table <- familiar:::..vimp(good_model)
+  vimp_table <- familiar:::get_vimp_table(good_model)
   
   # Expect that the vimp table has two rows.
   testthat::expect_equal(nrow(vimp_table), 13)
@@ -114,7 +114,7 @@ testthat::test_that("Extreme gradient boosting regression model can train on wid
   testthat::expect_equal(familiar:::model_is_trained(wide_model), TRUE)
   
   # Variable importance table is present.
-  testthat::expect_equal(familiar:::is_empty(familiar:::..vimp(wide_model)), FALSE)
+  testthat::expect_equal(familiar:::is_empty(familiar:::get_vimp_table(wide_model)), FALSE)
   
   # Valid predictions.
   testthat::expect_equal(familiar:::any_predictions_valid(familiar:::.predict(wide_model, wide_data), outcome_type=wide_data@outcome_type), TRUE)
@@ -171,7 +171,7 @@ testthat::test_that("Extreme gradient boosting regression model trained correctl
 testthat::test_that("Extreme gradient boosting regression model has variable importance", {
   
   # Extract the variable importance table.
-  vimp_table <- familiar:::..vimp(good_model)
+  vimp_table <- familiar:::get_vimp_table(good_model)
   
   # Expect that the vimp table has two rows.
   testthat::expect_equal(nrow(vimp_table), 10)
@@ -191,7 +191,7 @@ testthat::test_that("Extreme gradient boosting regression model can train on wid
   testthat::expect_equal(familiar:::model_is_trained(wide_model), TRUE)
   
   # Variable importance table is present.
-  testthat::expect_equal(familiar:::is_empty(familiar:::..vimp(wide_model)), FALSE)
+  testthat::expect_equal(familiar:::is_empty(familiar:::get_vimp_table(wide_model)), FALSE)
   
   # Valid predictions.
   testthat::expect_equal(familiar:::any_predictions_valid(familiar:::.predict(wide_model, wide_data), outcome_type=wide_data@outcome_type), TRUE)
@@ -247,7 +247,7 @@ testthat::test_that("Extreme gradient boosting regression model trained correctl
 testthat::test_that("Extreme gradient boosting regression model has variable importance", {
   
   # Extract the variable importance table.
-  vimp_table <- familiar:::..vimp(good_model)
+  vimp_table <- familiar:::get_vimp_table(good_model)
   
   # Expect that the vimp table has two rows.
   testthat::expect_equal(nrow(vimp_table), 8)
@@ -267,7 +267,7 @@ testthat::test_that("Extreme gradient boosting regression model can train on wid
   testthat::expect_equal(familiar:::model_is_trained(wide_model), TRUE)
   
   # Variable importance table is present.
-  testthat::expect_equal(familiar:::is_empty(familiar:::..vimp(wide_model)), FALSE)
+  testthat::expect_equal(familiar:::is_empty(familiar:::get_vimp_table(wide_model)), FALSE)
   
   # Valid predictions.
   testthat::expect_equal(familiar:::any_predictions_valid(familiar:::.predict(wide_model, wide_data), outcome_type=wide_data@outcome_type), TRUE)
@@ -324,7 +324,7 @@ testthat::test_that("Extreme gradient boosting regression model trained correctl
 testthat::test_that("Extreme gradient boosting regression model has variable importance", {
   
   # Extract the variable importance table.
-  vimp_table <- familiar:::..vimp(good_model)
+  vimp_table <- familiar:::get_vimp_table(good_model)
   
   # Expect that the vimp table has two rows.
   testthat::expect_equal(nrow(vimp_table), 4)
@@ -344,7 +344,7 @@ testthat::test_that("Extreme gradient boosting regression model can train on wid
   testthat::expect_equal(familiar:::model_is_trained(wide_model), TRUE)
   
   # Variable importance table is present.
-  testthat::expect_equal(familiar:::is_empty(familiar:::..vimp(wide_model)), FALSE)
+  testthat::expect_equal(familiar:::is_empty(familiar:::get_vimp_table(wide_model)), FALSE)
   
   # Valid predictions.
   testthat::expect_equal(familiar:::any_predictions_valid(familiar:::.predict(wide_model, wide_data), outcome_type=wide_data@outcome_type), TRUE)
@@ -410,7 +410,7 @@ testthat::test_that("Extreme gradient boosting regression model trained correctl
 testthat::test_that("Extreme gradient boosting regression model has variable importance", {
   
   # Extract the variable importance table.
-  vimp_table <- familiar:::..vimp(good_model)
+  vimp_table <- familiar:::get_vimp_table(good_model)
   
   # Expect that the vimp table has three rows.
   testthat::expect_equal(nrow(vimp_table), 3)
@@ -430,7 +430,7 @@ testthat::test_that("Extreme gradient boosting regression model can train and pr
   testthat::expect_equal(familiar:::model_is_trained(wide_model), TRUE)
   
   # Variable importance table is present.
-  testthat::expect_equal(familiar:::is_empty(familiar:::..vimp(wide_model)), FALSE)
+  testthat::expect_equal(familiar:::is_empty(familiar:::get_vimp_table(wide_model)), FALSE)
   
   # Valid predictions are present.
   testthat::expect_equal(familiar:::any_predictions_valid(familiar:::.predict(wide_model, wide_data), outcome_type=wide_data@outcome_type), TRUE)
