@@ -5,8 +5,7 @@ testthat::skip_on_cran()
 
 familiar:::test_hyperparameter_optimisation(vimp_methods=familiar:::.get_available_correlation_vimp_methods(show_general=TRUE),
                                             debug=FALSE,
-                                            parallel=FALSE,
-                                            no_hyperparameters=TRUE)
+                                            parallel=FALSE)
 familiar:::test_all_vimp_methods(familiar:::.get_available_correlation_vimp_methods(show_general=FALSE))
 familiar:::test_all_vimp_methods_parallel(familiar:::.get_available_correlation_vimp_methods(show_general=FALSE))
 
